<?php

namespace Drupal\mcapi\Entity\Access;

use Drupal\mcapi\Entity\WalletInterface;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\mcapi\Mcapi;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Defines an access controller option for the mcapi_wallet entity.
 *
 */
class WalletAccessControlHandler extends EntityAccessControlHandler {

  private $done = FALSE;

  /**
   * @var RouteMatchInterface
   */
  private $routeMatch = FALSE;

  /**
   * @todo inject $routematch
   */
  public function __construct(EntityTypeInterface $entity_type) {
    parent::__construct($entity_type);
    $this->routeMatch = \Drupal::routeMatch();

  }

  /**
   * {@inheritdoc}
   */
  public function createAccess($entity_bundle = NULL, AccountInterface $account = NULL, array $context = [], $return_as_object = FALSE) {
    // In the unlikely event that a page might try to display two create wallet buttons, this might reduce confusion.
    if($this->done == FALSE) {
      $this->done = TRUE;
      // This fetches the entity we are viewing - the would-be holder of the
      // wallet we would add.
      $params = $this->routeMatch->getParameters()->all();
      $holder = reset($params);
      // Quick-check for users to add their own wallets.
      if (($account->hasPermission('create own wallets') && $holder->getEntityTypeId() == 'user' && $account->id() == $holder->id()) or $account->hasPermission('manage mcapi')) {
        return $this->lessThanMaxWalletsExisting($holder, $account);
      }
      return AccessResult::forbidden("This user cannot create a $holder_type wallet")->addCacheableDependency($account);
    }
    return AccessResult::forbidden('Only one wallet create button per page');
  }

  /**
   *
   * @param ContentEntityInterface $holder
   * @param AccountInterface $account
   * @return AccessResult
   */
  private function lessThanMaxWalletsExisting(ContentEntityInterface $holder, AccountInterface $account) : AccessResult {
    $owned_wallet_ids = \Drupal::entityTypeManager()
      ->getStorage('mcapi_wallet')->getQuery()
      ->condition('holder_entity_type', $holder->getEntityTypeId())
      ->condition('holder_entity_id', $holder->id())
      ->execute();
    $max = Mcapi::maxWalletsOfBundle($holder);
    if (count($owned_wallet_ids) < $max) {
      $result = AccessResult::allowed();
    }
    else {
      $result = AccessResult::forbidden($holder->getEntityTypeId().' '. $holder->id() ." already has $max wallets.");
    }
    return $result->addCacheableDependency($account);
  }

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $op, AccountInterface $account) {
    $this->prepareUser($account);
    if ($account->hasPermission('manage mcapi')) {
      if ($op == 'delete') {
        // Don't cache
        return AccessResult::allowedIf($entity->isVirgin());
      }
      // Includes user 1.
      return AccessResult::allowed()->cachePerPermissions();
    }
    elseif ($op == 'view') {
      if ($entity->getOwnerId() == $account->id()) {
        return AccessResult::allowed()->cachePerUser();
      }
      return AccessResult::allowedIfhasPermission($account, 'view all transactions')
        ->cachePerPermissions();
    }
    elseif ($this->controlsWallet($entity, $account)) {
      if ($op == 'delete') {
        // Don't cache
        return AccessResult::allowedIf($entity->isVirgin());
      }
      return AccessResult::allowed()->cachePerUser();
    }
//    elseif ($op == 'transactions') {// This isn't used
//      if ($account->hasPermission('view all transactions')) {
//        return AccessResult::allowed()->cachePerPermissions();
//      }
//      elseif ($entity->public->value == TRUE) {
//        return AccessResult::allowed()->addCacheableDependency($entity);
//      }
//      return AccessResult::neutral()->addCacheableDependency($entity);
//    }
    elseif ($op == 'update') {
      return AccessResult::forbidden()->cachePerUser();
    }
  }

  /**
   * Check if the given entity holds the given wallet.
   *
   * @param WalletInterface $wallet
   * @param AccountInterface $account
   *
   * @return boolean
   *   TRUE if the $account controls the $entity
   */
  private function controlsWallet(WalletInterface $wallet, AccountInterface $account) : bool {
    return $wallet->getOwnerId() == $account->id();
  }

}
